/**
 * @author Joel Espinosa Longi
 * @licencia LGPL - http://www.gnu.org/licenses/lgpl.html
 */

var editor = (function(editor) {

  /**
   *
   */
  editor.ContentManager = { };

  /**
   *
   */
  editor.ContentManager.closeFile = function() {
    // clear the console logs
    console.clear();

    var tmpElems = [];
    var elems = document.querySelectorAll("iframe");
    for (var i=0, l=elems.length; i<l; i++) {
      tmpElems.push( elems[i] );
    }
    for (var i=0, l=tmpElems.length; i<l; i++) {
      tmpElems[i].win.close(true);
      tmpElems[i].win = null;
      tmpElems[i].parentNode.removeChild(tmpElems[i]);
    }

    tmpElems = [];
    elems = document.querySelectorAll(".descartes_edit_button");
    for (var i=0, l=elems.length; i<l; i++) {
      tmpElems.push( elems[i] );
    }
    for (var i=0, l=tmpElems.length; i<l; i++) {
      tmpElems[i].parentNode.removeChild(tmpElems[i]);
    }

  }

  /**
   *
   */
  editor.ContentManager.openFile = function(filename) {
console.log(filename);

    // ToDo: check the file if is of the correct type
    // the beatify is necessary because the for some reasons known directly some times mess with the head tags
    var contents = beautify_html( editor.File.open(filename), { indent_size:0, max_preserve_newlines:0, wrap_line_length:0, unformatted:["script"] } );
    // parse the content and create a dom object
    editor.contentDoc = (new DOMParser()).parseFromString(contents, "text/html");

    // all the descartes macros scripts
    editor.descMacros = [];
    editor.descMacrosText = [];

    // get all scripts that contains a referenc to descartes-min.js
    var jsScripts = editor.contentDoc.querySelectorAll("script");
    var desminScript = [];
    var tmpSrc;
    for (var i=0, l=jsScripts.length; i<l; i++) {
      tmpSrc = jsScripts[i].getAttribute("src");
      if (tmpSrc && tmpSrc.match("descartes-min.js")) {
        desminScript.push(jsScripts[i]);
      }

      if (jsScripts[i].getAttribute("type") == "descartes/macro") {
        editor.descMacros.push(jsScripts[i]);
        editor.descMacrosText.push(jsScripts[i].outerHTML);
      }
    }

    editor.descMinType = null;
    // remove all the scripts that reference descartes-min.js and get the type of the descartes-min.js reference
    for (var i=0, l=desminScript.length; i<l; i++) {
      if (!editor.descMinType) {
        // find which type of descartes-min.js has
        // ToDo: maybe if has a diferente inclusion dont change it
        tmpSrc = desminScript[i].getAttribute("src");
        if (tmpSrc.match("http://arquimedes.matem.unam.mx/Descartes5/lib/descartes-min.js")) {
          editor.descMinType = "internet";
        }
        else if (tmpSrc.match("../lib/descartes-min.js")) {
          editor.descMinType = "proyect";
        }
        else {
          editor.descMinType = "portable";
        }
      }
      desminScript[i].parentNode.removeChild(desminScript[i]);
    }
    editor.descMinType = editor.descMinType || "portable";

    // add a descartes-min.js script
    editor.descMinScript = document.createElement("script");
    editor.descMinScript.setAttribute("type", "text/javascript");
    if (editor.descMinType == "internet") {
      tmpSrc = "http://arquimedes.matem.unam.mx/Descartes5/lib/descartes-min.js";
    }
    else if (editor.descMinType == "proyect") {
      tmpSrc = "../lib/descartes-min.js";
    }
    else {
      tmpSrc = "lib/descartes-min.js";
    }
    editor.descMinScript.setAttribute("src", tmpSrc);
    editor.contentDoc.head.appendChild(editor.descMinScript);

    // find and remove all the meta tags with http-equiv="Content-Type"
    var metatags = editor.contentDoc.querySelectorAll("meta");
    var metatagsArrary = [];
    var tmpAttr;
    for (var i=0, l=metatags.length; i<l; i++) {
      // if the meta tag is of the type <meta http-equiv="content-type" content="text/html; charset=UTF-8">
      tmpAttr = metatags[i].getAttribute("http-equiv");
      if (tmpAttr && (tmpAttr.toLowerCase() === "content-type")) {
        metatagsArrary.push(metatags[i]);
      }
      // if the meta tag is of the type <meta charset="UTF-8">
      tmpAttr = metatags[i].getAttribute("charset");
      if (tmpAttr) {
        metatagsArrary.push(metatags[i]);
      }
    }
    // remove the tags
    for (var i=0, l=metatagsArrary.length; i<l; i++) {
      metatagsArrary[i].parentNode.removeChild(metatagsArrary[i]);
    }
    // add a new meta tag
    var meta = document.createElement("meta");
    meta.setAttribute("http-equiv", "content-type");
    meta.setAttribute("content", "text/html; charset=UTF-8");
    editor.contentDoc.head.insertBefore(meta, editor.contentDoc.head.firstChild);

    // query all applets
    // editor.applets = editor.contentDoc.querySelectorAll("applet");
    // if (editor.applets.length == 0) {
      editor.applets = editor.contentDoc.querySelectorAll("ajs");
    // }

    // init the iframes array to hold the temporary iframes with the descartes scenes
    editor.iframes = [];
    // create a temporal iframe for every applet
    for (var i=0, l=editor.applets.length; i<l; i++) {
      // ToDo: change applet with ajs tags, the attributes has to be selected, not all is copied, all is needed the size of the app
      // var tmp = document.createElement("ajs");
      // copyAttributes(editor.applets[i], tmp);
      // tmp.innerHTML = editor.applets[i].innerHTML;
      // console.log(tmp)      
      editor.iframes.push( { applet: editor.applets[i],
                             iframe: this.createNewIframe(editor.applets[i], filename)
                           } 
                          );
    }

  }

  /**
   *
   */
  function copyAttributes(src, dst) {
    for (var i=0, l=src.attributes.length; i<l; i++) {
      dst.setAttribute(src.attributes[i].name, src.attributes[i].value);
    }

  }

  /** 
   *
   */
  editor.ContentManager.createNewIframe = function(applet, filename) {
    var iframe = document.createElement("iframe");
    iframe.src = "about:blank";
    iframe.width = applet.getAttribute("width");
    iframe.height = applet.getAttribute("height");
    iframe.frameBorder = 0;
    iframe.scrolling = "no";
    document.body.appendChild(iframe);
    var doc = iframe.contentWindow.document;

    var divEdit = document.createElement("div");
    divEdit.innerHTML = "E";
    divEdit.setAttribute("class", "descartes_edit_button");
    document.body.appendChild(divEdit);
    divEdit.addEventListener("click", editIframe);

    var content = "<!DOCTYPE html>" +
     "<head><title></title>" +
     "<script type='text/javascript' src='lib/descartes-min.js'></script>" +
     "<base href='" + filename + "'>" +
     editor.contentDoc.head.innerHTML +
     "</head>" +
     "<body style='background:#fff;'>" +
     applet.outerHTML +
     editor.descMacrosText.join("\n\n") +
     "</body></html>";

    doc.open("text/htmlreplace");
    doc.write(content);
    doc.close();

    iframe.contentWindow.document.addEventListener("mousedown", function(evt) {
      // prevent the external space to show when right click the space
      if (evt.which === 3) {
        evt.stopPropagation();
        evt.preventDefault();
      }
    }, true);

    // add a model to hold the configuration information
    iframe.model = new editor.Model(applet);

    iframe.win = gui.Window.open("paramEditor.html", {
      title: "Configuración",
      position: "center",
      width: 800,
      min_width: 800,
      height: 600,
      min_height: 600,
      toolbar: false,
      focus: true,
      show: false
    });
    iframe.win.on("close", function() {
      iframe.win.hide();
    });
    iframe.win.on("loaded", function() {
      console.log("leido")
      iframe.win.window.paramEditor.setParams(iframe.model);
      // iframe.win.showDevTools();
      // console.log(iframe.win.window);
      // iframe.win.window
    });

    function editIframe(evt) {
      // iframe.win.window.document.getElementById("container").innerHTML = "bla";

      // when finish the configuration, then show the window
      iframe.win.show();
      iframe.win.focus();
    }

    return iframe;
  }


  /** 
   *
   */
  editor.ContentManager.saveFile = function(filename) {
    // remove the descartes macros script to prevent a bad format
    this.cleanDescMacros();
    editor.File.save(filename,
                     "<!DOCTYPE html>\n<html>\n" + 
                     beautify_html( editor.contentDoc.querySelector("html").innerHTML, 
                       { indent_size:2, 
                         max_preserve_newlines:0, 
                         wrap_line_length:0,
                         extra_liners:["head","body","/html","div","p"]
                       }
                     ).replace("</body>", "") +
                     editor.descMacrosText.join("\n\n") +
                     "\n\n</body>\n</html>"
    );
    // return the macros elements to the doc
    this.restoreDescMacros();
  }

  /**
   *
   */
  editor.ContentManager.cleanDescMacros = function() {
    // remove all the macro scripts
    for (var i=0, l=editor.descMacros.length; i<l; i++) {
      editor.descMacros[i].parentNode.removeChild(editor.descMacros[i]);
      editor.descMacrosText[i] = editor.descMacros[i].outerHTML;
    }
  }

  /**
   *
   */
  editor.ContentManager.restoreDescMacros = function() {
    // restore all the macro scripts
    for (var i=0, l=editor.descMacros.length; i<l; i++) {
      editor.contentDoc.body.appendChild(editor.descMacros[i]);
    }
  }

  return editor;
})(editor || {});
