/**
 * @author Joel Espinosa Longi
 * @licencia LGPL - http://www.gnu.org/licenses/lgpl.html
 */

var editor = (function(editor) {

  /**
   *
   */
  editor.configGUI = function() {
    this.initMenu();
    this.initHiddenInput();
  }

  /**
   *
   */
  editor.initHiddenInput = function() {
    // open input
    document.getElementById("open_input").addEventListener("change", function(evt) {
      editor.Controller.exec("openFile", this.value);
      this.value = "";
    });

    // save input
    document.getElementById("save_input").addEventListener("change", function(evt) {
      editor.Controller.exec("saveFile", this.value);
      this.value = "";
    });
  }

  /**
   * init the node webkit window menu
   */
  editor.initMenu = function() {
    var self = this;

    // Load native UI library
    var guiWindow = gui.Window.get();
// guiWindow.showDevTools();

    // Create an empty menu
    var menu = new gui.Menu();

    // Get the current window
    var win = gui.Window.get();

    // Create a menubar for window menu
    var menubar = new gui.Menu({ type: "menubar" });

    // Create a menuitem
    var file_menu = new gui.Menu();

    var file_menu_new = new gui.MenuItem({ 
      label: "Nuevo",
      click: function() {
        editor.Controller.exec("newFile");
      }
    });
    var file_menu_open = new gui.MenuItem({ 
      label: "Abrir",
      click: function() {
        self.clickInput("open_input");
      }
    });
    var file_menu_save = new gui.MenuItem({ 
      label: "Guardar", 
      click: function() {
        // if has a filename, then save the file
        if (editor.filename) {
          editor.Controller.exec("saveFile", editor.filename);
        }
        // if dont have filename, then show a save dialog
        else if (editor.content) {
          self.clickInput("save_input");
        }
      }
    });
    var file_menu_save_as = new gui.MenuItem({ 
      label: "Guardar como...", 
      click: function() {
        // show a save dialog
        self.clickInput("save_input");
      }
    });
    var file_menu_close = new gui.MenuItem({ 
      label: "Cerrar", 
      click: function() {
        editor.Controller.exec("closeFile");
      }
    });
    var file_menu_exit = new gui.MenuItem({ 
      label: "Salir", 
      click: function() {
        guiWindow.close();
        gui.App.closeAllWindows()
      }
    });

    file_menu.append(file_menu_new);
    file_menu.append(file_menu_open);
    file_menu.append(file_menu_save);
    file_menu.append(file_menu_save_as);
    file_menu.append(file_menu_close);
    file_menu.append(file_menu_exit);

    // Create menuitem
    var edit_menu = new gui.Menu();
    var edit_menu_algo = new gui.MenuItem({ 
      label: "...",
      click: function() {
        console.log("algo");
        // editor.Controller.exec("algo");
      }
    });

    edit_menu.append(edit_menu_algo);

    // add the menu items to the menu
    menubar.append(new gui.MenuItem({ label: "Archivo", submenu: file_menu}));
    menubar.append(new gui.MenuItem({ label: "Edición", submenu: edit_menu}));

    //assign the menubar to window menu
    win.menu = menubar;
  }


  /**
   * do a click in a DOM object with some id
   */
  editor.clickInput = function(id) {
    var event = document.createEvent('MouseEvents');
    event.initMouseEvent('click');
    document.getElementById(id).dispatchEvent(event);
  }
 
  /**
   * short cut keys
   */
  editor.keyupFunction = function(evt) {
    // new control
    if (evt.keyCode == 'N'.charCodeAt(0) && evt.ctrlKey) {
      editor.clickInput("new_btn");
    }
    // open control
    else if (evt.keyCode == 'O'.charCodeAt(0) && evt.ctrlKey) {
      editor.clickInput("open_input");
    }
    // save control
    else if (evt.keyCode == 'S'.charCodeAt(0) && evt.ctrlKey) {
      editor.clickInput("save_btn");
    }
  }

  return editor;
})(editor || {});