/**
 * @author Joel Espinosa Longi
 * @licencia LGPL - http://www.gnu.org/licenses/lgpl.html
 */

var editor = (function(editor) {

  /**
   *
   */
  editor.Model = function(applet) {
    this.data = {
      attributes: {
        width: applet.getAttribute("width") || 970,
        height: applet.getAttribute("height") || 550,
        size: "970x550"
      },
      spaces: [],
      controls: [],
      definitions: [],
      programs: [],
      graphics: [],
      graphics3D: [],
      animation: null
    };
    var children = applet.children;
    var children_name;
    var children_value;
    var babel_name;
    var babel_value;
    var buttoms_params;
    var tmpSplit;
    var tmpType;

// console.log(children)
    for (var i=0, l=children.length; i<l; i++) {
      children_name = children[i].name;
      children_value = children[i].value;
      babel_name = babel[children_name];

      // space definitions
      if (children_name.match(/^E_/)) {
        // console.log("space");
        // console.log(children_name, ":", children_value);
        this.data.spaces.push( this.newSpaceObject(this.split(children_value)) );
      }
      // control definitions
      else if (children_name.match(/^C_/)) {
        // console.log("control");
        // console.log(children_name, ":", children_value);
        this.data.controls.push( this.newControlObject(this.split(children_value)) );
      }
      // auxiliar definitions
      else if (children_name.match(/^A_/)) {
        // console.log("auxiliar");
        tmpSplit = this.split(children_value);
        tmpType = this.getTypeAux(tmpSplit);

        // console.log(children_name, ":", children_value, "(" + tmpType + ")");

        if ( (tmpType == "constant") || (tmpType == "event") || (tmpType == "algorithm")) { 
          this.data.programs.push( this.newProgramObject(tmpSplit, tmpType) );
        }
        else {
          this.data.definitions.push( this.newDefinitionObject(tmpSplit, tmpType) );
        }
      }
      // graphics definitions
      else if (children_name.match(/^G_/)) {
        // console.log("graphics");
        // console.log(children_name, ":", children_value);
        this.data.graphics.push( this.newGraphicObject(this.split(children_value)) );
      }
      // 3d graphics definitions
      else if (children_name.match(/^S_/)) {
        // console.log("3d graphics");
        // console.log(children_name, ":", children_value);
        this.data.graphics3D.push( this.newGraphic3DObject(this.split(children_value)) );
      }
      // animation definition
      else if (children_name.match(/^Anima/)) {
        // console.log("animation");
        // console.log(children_name, ":", children_value);
        this.data.animation = this.newAnimation(this.split(children_value));
      }
      // scene params
      else {
        // console.log("scene params");
        // console.log(children_name, "---", babel[children_name], ":", children_value, "---", babel[children_value]);
        if (babel_name) {
          // extra crap case
          if (babel_name == "Buttons") {
            buttoms_params = this.split(children_value);

            for (var bi=0, bl=buttoms_params.length; bi<bl; bi++) {

              babel_name = babel[buttoms_params[bi].name] || buttoms_params[bi].name;
              babel_value = babel[buttoms_params[bi].value] || buttoms_params[bi].value;


              this.data.attributes[babel_name] = babel_value;
            }
          }
          else {
            babel_value = babel[children_value] || children_value;
            if (babel_value) {
              this.data.attributes[babel_name] = babel_value;
            }
          }
        }
        else {
          console.log("no se encontro |", children_name, "|");
        }
      }

      // console.log(children_name);
    }

    // console.log(this.data)
  }

  /**
   *
   */
  editor.Model.prototype.newSpaceObject = function(values) {
    var tmpType = "R2";

    for(var i=0, l=values.length; i<l; i++) {
      if (babel[values[i].name] == "type") {
        tmpType = babel[values[i].value] || values[i].value;
      }
    }

    var obj;

    if (tmpType === "R2") {
      obj = {
        type:       "R2",
        id:         "E0",
        cID:        "",
        x:          "0",
        y:          "0",
        width:      "100%", 
        height:     "100%",
        drawif:     "1",
        fixed:      "false",
        scale:      "48",
        "O.x":      "0",
        "O.y":      "0",
        image:      "",
        bg_display: "topleft",
        background: "f0f8fa",
        net:        "b8c4c8",
        net10:      "889498",
        axes:       "405860",
        text:       "405860",
        numbers:    "false",
        x_axis:     "",
        y_axis:     "",
        sensitive_to_mouse_movements: "false"
      };
    }
    else if (tmpType === "R3") {
      obj = {
        type:       "R3",
        R3:         "true",
        id:         "E0",
        x:          "0",
        y:          "0",
        width:      "100%", 
        height:     "100%",
        drawif:     "1",
        fixed:      "false",
        scale:      "48",
        "O.x":      "0",
        "O.y":      "0",
        image:      "",
        bg_display: "topleft",
        background: "000000",
        render:     "order",
        split:      "false",
        sensitive_to_mouse_movements: "false"
      }
    }
    else {
      obj = {
        type:       "HTMLIFrame",
        id:         "E0",
        x:          "0",
        y:          "0",
        width:      "100%", 
        height:     "100%",
        drawif:     "1",
        image:      "",
        bg_display: "topleft",
        background: "f0f8fa",
        file:       ""
      }
    }

    for(var i=0, l=values.length; i<l; i++) {
      obj[babel[values[i].name]] = babel[values[i].value] || values[i].value;
    }

    return obj;
  }

  /**
   *
   */
  editor.Model.prototype.newControlObject = function(values) {
    var tmpType = "spinner";

    for(var i=0, l=values.length; i<l; i++) {
      if (babel[values[i].name] == "gui") {
        tmpType = babel[values[i].value] || values[i].value;
      }
    }

    var obj;

    // if the type is spinner
    if (tmpType == "spinner") {
      obj = { 
        id:         "n1",
        type:       "numeric",
        gui:        "spinner",
        region:     "south",
        space:      "E0",
        name:       "",
        expresion:  "(0,0,100,23)",
        value:      "0",
        incr:       "0.1",
        min:        "",
        max:        "",
        discrete:   "false",
        decimals:   "2",
        fixed:      "true",
        exponentialif: "",
        visible:    "true",
        action:     "",
        parameter:  "",
        drawif:     "",
        activeif:   ""
        // evaluate:   "false",
        // answer:     "",
        // weight:     ""
      };
    }
    // if the type is textfield
    else if (tmpType == "textfield") {
      obj = { 
        id:         "n1",
        type:       "numeric",
        gui:        "textfield",
        onlyText:   "false",
        region:     "south",
        space:      "E0",
        name:       "",
        expresion:  "(0,0,100,23)",
        value:      "0",
        incr:       "0.1",
        min:        "",
        max:        "",
        discrete:   "false",
        decimals:   "2",
        fixed:      "true",
        exponentialif: "",
        visible:    "true",
        action:     "",
        parameter:  "",
        drawif:     "",
        activeif:   "",
        evaluate:   "false",
        answer:     ""
        // weight:     ""
      };
    }
    // if the type is menu
    else if (tmpType == "menu") {
      obj = { 
        id:         "n1",
        type:       "numeric",
        gui:        "menu",
        region:     "south",
        space:      "E0",
        name:       "",
        expresion:  "(0,0,100,23)",
        value:      "0",
        decimals:   "2",
        fixed:      "true",
        exponentialif: "",
        visible:    "true",
        options:    "",
        action:     "",
        parameter:  "",
        drawif:     "",
        activeif:   ""
        // evaluate:   "false",
        // answer:     "",
        // weight:     ""
      };
    }
    // if the type is scrollbar
    else if (tmpType == "scrollbar") {
      obj = { 
        id:         "n1",
        type:       "numeric",
        gui:        "scrollbar",
        region:     "south",
        space:      "E0",
        name:       "",
        expresion:  "(0,0,100,23)",
        value:      "0",
        incr:       "0.1",
        min:        "0",
        max:        "100",
        discrete:   "false",
        decimals:   "2",
        fixed:      "true",
        exponentialif: "",
        visible:    "true",
        action:     "",
        parameter:  "",
        drawif:     "",
        activeif:   ""
        // evaluate:   "false",
        // answer:     "",
        // weight:     ""
      };
    }
    // if the type is button
    else if (tmpType == "button") {
      obj = { 
        id:         "n1",
        gui:        "button",
        type:       "numeric",
        region:     "south",
        space:      "E0",
        name:       "",
        expresion:  "(0,0,100,23)",
        value:      "0",
        visible:    "true",
        color:      "222222",
        colorInt:   "f0f8ff",
        bold:       "false",
        italics:    "false",
        underlined: "false",
        font_size:  "12",
        image:      "",
        action:     "",
        parameter:  "",
        drawif:     "",
        activeif:   ""
      };
    }
    // if the type is graphic
    else if (tmpType == "graphic") {
      obj = { 
        id:         "g1",
        type:       "graphic",
        space:      "E0",
        color:      "222222",
        colorInt:   "f0f8ff",
        size:       "4",
        expresion:  "(0,0)",
        constraint: "",
        image:      "",
        drawif:     "",
        activeif:   ""
      };           
    }
    // if the type is text
    else if (tmpType == "text") {
      obj = { 
        id:         "t1",
        type:       "text",
        space:      "E0",
        expresion:  "(0,0)",
        text:       "",
        answer:     "",
        drawif:     "",
        activeif:   ""
      };
    }
    // if the type is audio
    else if (tmpType == "audio") {
      obj = { 
        id:         "a1",
        type:       "audio",
        space:      "E0",
        expresion:  "(0,0)",
        drawif:     "",
        file:       ""
      };
    }
    // if the type is video
    else {
      obj = { 
        id:         "v1",
        type:       "video",
        space:      "E0",
        expresion:  "(0,0)",
        drawif:     "",
        file:       ""
      };
    }

    for(var i=0, l=values.length; i<l; i++) {
      obj[babel[values[i].name]] = babel[values[i].value] || values[i].value;
    }

    return obj;
  }

  /**
   *
   */
  editor.Model.prototype.newDefinitionObject = function(values, tmpType) {
    var obj = null;

    // if the type is variable
    if (tmpType == "variable") {
      obj = { 
        id:        "v1",
        expresion: "0", 
        type:      "variable"
      };
    }
    // if the type is function
    else if (tmpType == "function") {
      obj = { 
        id:        "f(x)",
        algorithm: "false",
        expresion: "x",
        range:     "",
        local:     "",
        init:      "",
        doExpr:    "",
        whileExpr: "",
        type:      "function"
      };
    }
    // if the type is array
    else if (tmpType == "array") {
      obj = { 
        id:        "V1",
        array:     "true",
        evaluate:  "onlyOnce",
        size:      "3",
        expresion: "V3[0]=0;V3[1]=0;V3[2]=0",
        file:      "",
        type:      "array"
      };
    }
    // if the type is matrix
    else if (tmpType == "matrix") {
      obj = { 
        id:        "M1",
        matrix:    "true",
        evaluate:  "onlyOnce",
        columns:   "3",
        rows:      "3",
        expresion: "M4[0,0]=0;M4[1,0]=0;M4[2,0]=0;M4[0,1]=0;M4[1,1]=0;M4[2,1]=0;M4[0,2]=0;M4[1,2]=0;M4[2,2]=0;", 
        type:      "matrix"
      };
    }

    if (obj) {
      for(var i=0, l=values.length; i<l; i++) {
        obj[babel[values[i].name]] = babel[values[i].value] || values[i].value;
      }
    }
   
    return obj;
  }

  /**
   *
   */
  editor.Model.prototype.newProgramObject = function(values, tmpType) {
    var obj = null;

    // if the type is constant
    if (tmpType == "constant") {
      obj = { 
        id:        "c1",
        expresion: "0",
        evaluate:  "onlyOnce", 
        constant:  "true",
        type:      "constant"
      };
    }
    // if the type is event
    else if (tmpType == "event") {
      obj = { 
        id:        "e1",
        "event":   "true",
        condition: "",
        action:    "",
        parameter: "",
        execution: "",
        type:      "event"
      };
    }
    // if the type is algorithm
    else if (tmpType == "algorithm") {
      obj = { 
        id:        "A1",
        algorithm: "true",
        evaluate:  "onlyOnce",
        init:      "",
        doExpr:    "",
        whileExpr: "", 
        type:      "algorithm"
      };
    }

    if (obj) {
      for(var i=0, l=values.length; i<l; i++) {
        obj[babel[values[i].name]] = babel[values[i].value] || values[i].value;
      }
    }
   
    return obj;
  }

  /**
   *
   */
  editor.Model.prototype.newGraphicObject = function(values) {
    var tmpType;

    for(var i=0, l=values.length; i<l; i++) {
      if (babel[values[i].name] == "type") {
        tmpType = babel[values[i].value] || values[i].value;
      }
    }

    var obj;

    // if the type is equation
    if (tmpType == "equation") {
      obj = { 
        space:           "E0",
        type:            "equation",
        background:      "false",
        color:           "20303a",
        drawif:          "",
        abs_coord:       "false",
        expresion:       "y=x",
        trace:           "false",
        useFamily:       "false",
        family:          "",
        family_interval: "",
        family_steps:    "",
        fillP:           "",
        fillM:           "",
        width:           "1",
        visible:         "false",
        editable:        "false",
        info:            ""
      };
    }
    // if the type is curve
    else if (tmpType == "curve") {
      obj = { 
        space:           "E0",
        type:            "curve",
        background:      "false",
        color:           "20303a",
        drawif:          "",
        abs_coord:       "false",
        expresion:       "(t,t)",
        trace:           "false",
        useFamily:       "false",
        family:          "",
        family_interval: "",
        family_steps:    "",
        parameter:       "t",
        parameter_interval: "[0,1]",
        parameter_steps: "8",
        fill:            "",
        width:           "1",
        visible:         "false",
        editable:        "false",
        info:            ""
      };
    }
    // if the type is sequence
    else if (tmpType == "sequence") {
      obj = { 
        space:           "E0",
        type:            "sequence",
        background:      "false",
        color:           "20303a",
        drawif:          "",
        abs_coord:       "false",
        expresion:       "(n,1/n)",
        trace:           "false",
        useFamily:       "false",
        family:          "",
        family_interval: "",
        family_steps:    "",
        size:            "2",
        visible:         "false",
        editable:        "false",
        range:           "[1,100]",
        info:            ""
      };
    }
    // if the type is point
    else if (tmpType == "point") {
      obj = { 
        space:           "E0",
        type:            "point",
        background:      "false",
        color:           "20303a",
        drawif:          "",
        abs_coord:       "false",
        expresion:       "(0,0)",
        trace:           "false",
        useFamily:       "false",
        family:          "",
        family_interval: "",
        family_steps:    "",
        text:            "",
        decimals:        "2",
        fixed:           "true",
        size:            "2",
        font:            "Monospaced,PLAIN,15",
        info:            ""
      };
    }
    // if the type is segment
    else if (tmpType == "segment") {
      obj = { 
        space:           "E0",
        type:            "segment",
        background:      "false",
        color:           "20303a",
        drawif:          "",
        abs_coord:       "false",
        expresion:       "(0,0)(1,1)",
        trace:           "false",
        useFamily:       "false",
        family:          "",
        family_interval: "",
        family_steps:    "",
        text:            "",
        decimals:        "2",
        fixed:           "true",
        size:            "2",
        width:           "1",
        font:            "Monospaced,PLAIN,15",
        info:            ""
      };
    }
    // if the type is arrow
    else if (tmpType == "arrow") {
      obj = { 
        space:           "E0",
        type:            "arrow",
        background:      "false",
        color:           "20303a",
        drawif:          "",
        abs_coord:       "false",
        expresion:       "(0,0)(1,1)",
        trace:           "false",
        useFamily:       "false",
        family:          "",
        family_interval: "",
        family_steps:    "",
        text:            "",
        decimals:        "2",
        fixed:           "true",
        width:           "5",
        size:            "",
        spear:           "8",
        arrow:           "ee0022",
        font:            "Monospaced,PLAIN,15",
        info:            ""
      };
    }
    // if the type is polygon
    else if (tmpType == "polygon") {
      obj = { 
        space:           "E0",
        type:            "polygon",
        background:      "false",
        color:           "20303a",
        drawif:          "",
        abs_coord:       "false",
        expresion:       "(0,0)(1,1)(2,-1)",
        trace:           "false",
        useFamily:       "false",
        family:          "",
        family_interval: "",
        family_steps:    "",
        fill:            "",
        width:           "1",
        info:            ""
      };
    }
    // if the type is arc
    else if (tmpType == "arc") {
      obj = { 
        space:           "E0",
        type:            "arc",
        background:      "false",
        color:           "20303a",
        drawif:          "",
        abs_coord:       "false",
        center:          "(0,0)",
        radius:          "1",
        init:            "0",
        end:             "90",
        vectors:         "false",
        trace:           "false",
        useFamily:       "false",
        family:          "",
        family_interval: "",
        family_steps:    "",
        text:            "",
        decimals:        "2",
        fixed:           "true",
        fill:            "",
        width:           "1",
        font:            "Monospaced,PLAIN,15",
        info:            ""
      };
    }
    // if the type is fill
    else if (tmpType == "fill") {
      obj = { 
        space:           "E0",
        type:            "fill",
        background:      "false",
        color:           "20303a",
        drawif:          "",
        abs_coord:       "false",
        expresion:       "(0,0)",
        trace:           "false",
        useFamily:       "false",
        family:          "",
        family_interval: "",
        family_steps:    "",
        info:            ""
      };
    }
    // if the type is text
    else if (tmpType == "text") {
      obj = { 
        space:           "E0",
        type:            "text",
        background:      "false",
        color:           "20303a",
        drawif:          "",
        expresion:       "[20,20]",
        trace:           "false",
        useFamily:       "false",
        family:          "",
        family_interval: "",
        family_steps:    "",
        text:            "",
        decimals:        "2",
        fixed:           "true",
        width:           "1",
        border:          "",
        font:            "Monospaced,PLAIN,15",
        info:            ""
      };
    }
    // if the type is image
    else if (tmpType == "image") {
      obj = { 
        space:           "E0",
        type:            "image",
        background:      "false",
        drawif:          "",
        abs_coord:       "false",
        expresion:       "(0,0)",
        trace:           "false",
        useFamily:       "false",
        family:          "",
        family_interval: "",
        family_steps:    "",
        file:            "",
        inirot:          "0",
        opacity:         "0",
        info:            ""
      };
    }
    // if the type is macro
    else if (tmpType == "macro") {
      obj = { 
        space:           "E0",
        type:            "macro",
        background:      "false",
        drawif:          "",
        abs_coord:       "false",
        name:            "mac1",
        expresion:       "",
        useFamily:       "false",
        family:          "",
        family_interval: "",
        family_steps:    "",
        inirot:          "0",
        inipos:          "[0,0]",
        info:            ""
      };
    }

    for(var i=0, l=values.length; i<l; i++) {
      obj[babel[values[i].name]] = babel[values[i].value] || values[i].value;
    }

    return obj;
  }

  /**
   *
   */
  editor.Model.prototype.newGraphic3DObject = function(values) {
    var tmpType;

    for(var i=0, l=values.length; i<l; i++) {
      if (babel[values[i].name] == "type") {
        tmpType = babel[values[i].value] || values[i].value;
      }
    }

    var obj;

    // if the type is point
    if (tmpType == "point") {
      obj = { 
        name:            "",
        space:           "E0",
        type:            "point",
        background:      "false",
        color:           "eeffaa",
        backcolor:       "6090a0",
        drawif:          "",
        expresion:       "(0,0,0)",
        useFamily:       "false",
        family:          "",
        family_interval: "",
        family_steps:    "",
        inirot:          "(0,0,0)",
        inipos:          "(0,0,0)",
        endrot:          "(0,0,0)",
        endpos:          "(0,0,0)",
        split:           "false",
        text:            "",
        decimals:        "2",
        fixed:           "true",
        width:           "1"
      };

    }
    // if the type is segment, polygon, curve
    else if ( (tmpType == "segment") || (tmpType == "polygon") || (tmpType == "curve") ) {
      obj = { 
        name:            "",
        space:           "E0",
        type:            tmpType,
        background:      "false",
        color:           "eeffaa",
        drawif:          "",
        expresion:       "",
        useFamily:       "false",
        family:          "",
        family_interval: "",
        family_steps:    "",
        inirot:          "(0,0,0)",
        inipos:          "(0,0,0)",
        endrot:          "(0,0,0)",
        endpos:          "(0,0,0)",
        split:           "false",
        width:           "1",
        Nu:              "7"
      };

      if (tmpType == "segment") {
        obj.expresion = "(0,0,0)(1,1,1)";
      }
      else if (tmpType == "polygon") {
        obj.expresion = "(0,0,0)(1,0,0)(1,1,0)(1,1,1)";
      }
      else {
        obj.expresion = "x=cos(4*pi*u) y=sen(4*pi*u) z=2*u-1";
      }
    }
    // if the type is triangle
    else if ( (tmpType == "triangle") || (tmpType == "face") ) {
      obj = { 
        name:            "",
        space:           "E0",
        type:            tmpType,
        background:      "false",
        color:           "eeffaa",
        backcolor:       "6090a0",
        drawif:          "",
        expresion:       "",
        useFamily:       "false",
        family:          "",
        family_interval: "",
        family_steps:    "",
        inirot:          "(0,0,0)",
        inipos:          "(0,0,0)",
        endrot:          "(0,0,0)",
        endpos:          "(0,0,0)",
        split:           "false",
        edges:           "false",
        model:           "metal"
      };

      if (tmpType == "triangle") {
        obj.expresion = "(1,0,0)(0,1,0)(0,0,1)";
      }
      else {
        obj.expresion = "(0,0)(0,1)(1,1)";
      }      
    }
    // if the type is polireg
    else if (tmpType == "polireg") {
      obj = { 
        name:            "",
        space:           "E0",
        type:            "polireg",
        background:      "false",
        color:           "eeffaa",
        backcolor:       "6090a0",
        drawif:          "",
        expresion:       "Polireg",
        useFamily:       "false",
        family:          "",
        family_interval: "",
        family_steps:    "",
        inirot:          "(0,0,0)",
        inipos:          "(0,0,0)",
        endrot:          "(0,0,0)",
        endpos:          "(0,0,0)",
        split:           "false",
        edges:           "false",
        model:           "metal",
        width:           "2",
        length:          "2",
        Nu:              "7"
      };
    }
    // if the type is surface
    else if (tmpType == "surface") {
      obj = { 
        name:            "",
        space:           "E0",
        type:            "surface",
        background:      "false",
        color:           "eeffaa",
        backcolor:       "6090a0",
        drawif:          "",
        expresion:       "x=2*u-1 y=2*v-1 z=x^2-y^2",
        useFamily:       "false",
        family:          "",
        family_interval: "",
        family_steps:    "",
        inirot:          "(0,0,0)",
        inipos:          "(0,0,0)",
        endrot:          "(0,0,0)",
        endpos:          "(0,0,0)",
        split:           "false",
        edges:           "false",
        model:           "metal",
        Nu:              "7",
        Nv:              "7"
      };
    }
    // if the type is text
    else if (tmpType == "text") {
      obj = { 
        name:            "",
        space:           "E0",
        type:            "face",
        background:      "false",
        color:           "eeffaa",
        drawif:          "",
        expresion:       "[20,20]",
        useFamily:       "false",
        family:          "",
        family_interval: "",
        family_steps:    "",
        inirot:          "(0,0,0)",
        inipos:          "(0,0,0)",
        text:            "",
        decimals:        "2",
        fixed:           "true"
      };
    }
    // if the type is box
    else if (tmpType == "box") {
      obj = { 
        name:            "",
        space:           "E0",
        type:            "box",
        background:      "false",
        color:           "eeffaa",
        backcolor:       "6090a0",
        drawif:          "",
        expresion:       "Paralelep\u00edpedo",
        useFamily:       "false",
        family:          "",
        family_interval: "",
        family_steps:    "",
        inirot:          "(0,0,0)",
        inipos:          "(0,0,0)",
        endrot:          "(0,0,0)",
        endpos:          "(0,0,0)",
        split:           "false",
        edges:           "false",
        model:           "metal",
        width:           "2",
        length:          "2",
        height:          "2"
      };
    }
    // if the type is cube, tetrahedron, octahedron, dodecahedron, icosahedron
    else if ( (tmpType == "cube") || (tmpType == "tetrahedron") || (tmpType == "octahedron") || (tmpType == "dodecahedron") || (tmpType == "icosahedron") ) {
      obj = { 
        name:            "",
        space:           "E0",
        type:            tmpType,
        background:      "false",
        color:           "eeffaa",
        backcolor:       "6090a0",
        drawif:          "",
        expresion:       "",
        useFamily:       "false",
        family:          "",
        family_interval: "",
        family_steps:    "",
        inirot:          "(0,0,0)",
        inipos:          "(0,0,0)",
        endrot:          "(0,0,0)",
        endpos:          "(0,0,0)",
        split:           "false",
        edges:           "false",
        model:           "metal",
        width:           "2"
      };

      if (tmpType == "cube") {
        obj.expresion = "Cubo";
      }
      else if (tmpType == "tetrahedron") {
        obj.expresion = "Tetraedro";
      }
      else if (tmpType == "octahedron") {
        obj.expresion = "Octaedro";
      }
      else if (tmpType == "dodecahedron") {
        obj.expresion = "Dodecaedro";
      }
      else {
        obj.expresion = "Icosaedro";
      }
    }
    // if the type is sphere
    else if (tmpType == "sphere") {
      obj = { 
        name:            "",
        space:           "E0",
        type:            "sphere",
        background:      "false",
        color:           "eeffaa",
        backcolor:       "6090a0",
        drawif:          "",
        expresion:       "Esfera",
        useFamily:       "false",
        family:          "",
        family_interval: "",
        family_steps:    "",
        inirot:          "(0,0,0)",
        inipos:          "(0,0,0)",
        endrot:          "(0,0,0)",
        endpos:          "(0,0,0)",
        split:           "false",
        edges:           "false",
        model:           "metal",
        width:           "2",
        Nu:              "7",
        Nv:              "7"
      };
    }
    // if the type is ellipsoid, cone, cylinder
    else if ( (tmpType == "ellipsoid") || (tmpType == "cone") || (tmpType == "cylinder") ) {
      obj = { 
        name:            "",
        space:           "E0",
        type:            tmpType,
        background:      "false",
        color:           "eeffaa",
        backcolor:       "6090a0",
        drawif:          "",
        expresion:       "",
        useFamily:       "false",
        family:          "",
        family_interval: "",
        family_steps:    "",
        inirot:          "(0,0,0)",
        inipos:          "(0,0,0)",
        endrot:          "(0,0,0)",
        endpos:          "(0,0,0)",
        split:           "false",
        edges:           "false",
        model:           "metal",
        width:           "2",
        length:          "2",
        height:          "2",
        Nu:              "7",
        Nv:              "7"
      };

      if (tmpType == "ellipsoid") {
        obj.expresion = "Elipsoide";
      }
      else if (tmpType == "cone") {
        obj.expresion = "Cono";
      }
      else {
        obj.expresion = "Cilindro";
      }
    }
    // if the type is macro
    else if (tmpType == "macro") {
      obj = { 
        name:            "",
        space:           "E0",
        type:            "macro",
        background:      "false",
        drawif:          "",
        expresion:       "mi_macro",
        useFamily:       "false",
        family:          "",
        family_interval: "",
        family_steps:    "",
        inirot:          "(0,0,0)",
        inipos:          "(0,0,0)",
        endrot:          "(0,0,0)",
        endpos:          "(0,0,0)"
      };
    }

    for(var i=0, l=values.length; i<l; i++) {
      obj[babel[values[i].name]] = babel[values[i].value] || values[i].value;
    }

    return obj;
  }

  /**
   *
   */
  editor.Model.prototype.newAnimation = function(values) {
    var obj = {
      // Animation: "false",
      delay: "1000",
      controls: "false",
      auto: "false",
      loop: "false",
      init: "",
      doExpr: "",
      whileExpr: "" 
    }
 
    for(var i=0, l=values.length; i<l; i++) {
      obj[babel[values[i].name]] = babel[values[i].value] || values[i].value;
    }

    return obj;
  }





  /**
   *
   */
  editor.Model.prototype.getTypeAux = function(values) {
    var obj = {};
    for (var i=0,l=values.length; i<l; i++) {
      obj[babel[values[i].name]] = babel[values[i].value] || values[i].value;
    }

    // sequence
    if (obj.sequence) {
      return "sequence";
    }
    // constant
    else if (obj.constant) {
      return "constant";
    }
    // algorithm
    else if ((obj.algorithm) && (obj.evaluate)) {
      return "algorithm";
    }
    // vector
    else if ((obj.array) && (!obj.matrix) && (obj.id.charAt(obj.id.length-1) != ")")) {
      return "array";
    }
    // matrix
    else if ((obj.matrix) && (obj.id.charAt(obj.id.length-1) != ")")) {
      return "matrix";
    }
    // event
    else if ((obj.event) && (obj.id.charAt(obj.id.length-1) != ")")) {
      return "event";
    }
    else {
      // function
      if (obj.id.charAt(obj.id.length-1) === ")") {
        return "function";
      } 
      // variable
      else {
        return "variable";
      }
    }
  }

  /**
   * Removes single quotes in the value and divided into an array of parameters name and value pairs
   * @param {String} values the string to divided
   * @return {Array<Array<String>>} return the array of name and value pairs
   */
  editor.Model.prototype.split = function(values) {
    if (typeof(values) != "string") {
      return [];
    }

    values = values || "";
    values = values.replace(/\\'/g, "’");
    
    splitValues = [];
    pos = 0;
    i = 0;
    initToken = false;
    initPosToken = 0;
    endPosToken = 0;
    stringToken = false;
    valueToken = false;

    // traverse the string to split
    while (pos < values.length) {
      // ignoring the blank spaces if not started the identification of a token
      if ((values.charAt(pos) === " ") && (!initToken)) {
        pos++;
      }
      
      // find a character which is different from a blank space
      if ((values.charAt(pos) !== " ") && (!initToken)) {
        initToken = true;
        initPosToken = pos;
      }
      
      // values ​​are specified as a string
      if ((values.charAt(pos) === "=") && (values.charAt(pos+1) === "'") && (!stringToken)) {
        stringToken = true;
        
        splitValues[i] = [values.substring(initPosToken, pos)]

        initPosToken = pos+2;
        
        pos+=2;
      }
      
      if ((stringToken) && (values.charAt(pos) === "'")) {
        stringToken = false;
        
        initToken = false;
        
        splitValues[i].push(values.substring(initPosToken, pos));
        
        i++;
      }

      // values ​​are specified as a word sequence
      if ((values.charAt(pos) === "=") && (values.charAt(pos+1) !== "'") && (!stringToken)) {
        splitValues[i] = [values.substring(initPosToken, pos)]

        initPosToken = pos+1;
        
        pos++;

        // find the next space and equal sign
        tmpIndexEqual = (values.substring(pos)).indexOf("=");

        if (tmpIndexEqual === -1) {
          tmpIndexEqual = values.length;
          tmpIndexSpace = values.length;
        } 
        else {
          tmpIndexEqual += pos;

          tmpIndexSpace = values.substring(pos, tmpIndexEqual).lastIndexOf(" ");
          if (tmpIndexSpace === -1) {
            tmpIndexSpace = values.length;
          }
          else {
            tmpIndexSpace += pos;
          }
        }

        splitValues[i].push(values.substring(initPosToken, tmpIndexSpace));
        i++;
        initToken = false;

        pos = tmpIndexSpace;
      }

      pos++;
    }

    var newSplitValues = [];
    for (var i=0, l=splitValues.length; i<l; i++) {
      newSplitValues.push( { name:splitValues[i][0], value:splitValues[i][1] } );
    }

    return newSplitValues;
  }

  return editor;
})(editor || {});